@@ -36,7 +36,7 @@ else:
 
     class x_wmi(Exception):
         def __init__(self, info='', com_error=None):
-            super(x_wmi, self).__init__()
+            super(x_wmi, self).__init__(info)
             self.info = info
             self.com_error = com_error
 
@@ -193,6 +193,19 @@ class DNSZoneAlreadyExists(DNSException):
     msg_fmt = _("DNS Zone already exists: %(zone_name)s")
 
 
+class WMIJobFailed(HyperVException):
+    msg_fmt = _("WMI job failed with status %(job_state)s. "
+                "Error summary description: %(error_summ_desc)s. "
+                "Error description: %(error_desc)s "
+                "Error code: %(error_code)s.")
+
+    def __init__(self, message=None, **kwargs):
+        self.error_code = kwargs.get('error_code', None)
+        self.job_state = kwargs.get('job_state', None)
+
+        super(WMIJobFailed, self).__init__(message, **kwargs)
+
+
 class JobTerminateFailed(HyperVException):
     msg_fmt = _("Could not terminate the requested job(s).")
 
