@@ -42,7 +42,7 @@ from os_win.utils import pathutils
 LOG = logging.getLogger(__name__)
 
 # TODO(claudiub): remove the is_planned_vm argument from methods once it is not
-# used  anymore.
+# used anymore.
 
 
 class VMUtils(baseutils.BaseUtilsVirt):
@@ -158,6 +158,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
                     ['ElementName'],
                     VirtualSystemType=self._VIRTUAL_SYSTEM_TYPE_REALIZED)]
 
+    @_utils.not_found_decorator(
+        translated_exc=exceptions.HyperVVMNotFoundException)
     def get_vm_summary_info(self, vm_name):
         vmsettings = self._lookup_vm_check(vm_name)
 
@@ -225,7 +227,11 @@ class VMUtils(baseutils.BaseUtilsVirt):
             return vms[0]
 
     def vm_exists(self, vm_name):
-        return self._lookup_vm(vm_name) is not None
+        """Checks if the Realized VM with the given name exists on the host."""
+        # NOTE(claudiub): A planned VM and a realized VM cannot exist at the
+        # same time on the same host. The 2 types must be treated separately,
+        # thus, this will only check if the Realized VM exits.
+        return self._lookup_vm(vm_name, False) is not None
 
     def get_vm_id(self, vm_name):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
@@ -659,6 +665,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
                                                              Reason=reason)
         self._jobutils.check_ret_val(ret_val, None)
 
+    @_utils.retry_decorator(exceptions=exceptions.WMIJobFailed)
     def set_vm_state(self, vm_name, req_state):
         """Set the desired state of the VM."""
 
@@ -1090,7 +1097,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
         # NOTE(abalutoiu): new_boot_order will contain ROOT uppercase
         # in the device paths while old_boot_order will contain root
-        # lowercase, which will cause the tupple addition result to contain
+        # lowercase, which will cause the tuple addition result to contain
         # each device path twice because of the root lowercase and uppercase.
         # Forcing all the device paths to uppercase fixes the issue.
         new_boot_order = [x.upper() for x in new_boot_order]
